@@ -96,6 +96,11 @@ diat_epid <- function(resultLoad, maxDistTaxa = 2){
             vvalue = NA
             svalue = NA
           }
+        } else {
+          # length(spsplit[[1]]) =<1
+          #species not found, make everything NA
+          vvalue = NA
+          svalue = NA
         }
       }
       #records the final consensus value
