@@ -17,7 +17,7 @@
 #' }
 #' Sample data in the examples is taken from:
 #' \itemize{
-#' \item Nicolosi Gelis, María Mercedes; Cochero, Joaquín; Donadelli, Jorge; Gómez, Nora. 2020. "Exploring the use of nuclear alterations, motility and ecological guilds in epipelic diatoms as biomonitoring tools for water quality improvement in urban impacted lowland streams". Ecological Indicators, 110, 105951. https://doi.org/10.1016/j.ecolind.2019.105951
+#' \item Nicolosi Gelis, María Mercedes; Cochero, Joaquín; Donadelli, Jorge; Gómez, Nora. 2020. "Exploring the use of nuclear alterations, motility and ecological guilds in epipelic diatoms as biomonitoring tools for water quality improvement in urban impacted lowland streams". Ecological Indicators, 110, 105951. https://doi:10.1016/j.ecolind.2019.105951
 #' }
 #' @keywords ecology diatom bioindicator biotic
 #' @encoding UTF-8
@@ -25,7 +25,7 @@
 #' @export diat_loadData
 
 ########-------- FUNCTION TO LOAD FILE  --------------#########
-#### IN THIS SECTION WE READ THE MATRIX
+#### This function inputs the data and runs the species recognition
 ### INPUT: CSV file with the samples * abundances or relative abundances
 ### OUTPUTS: a dataframe with the species as matched against the database, with species in RA; Taxaincluded and Taxaexcluded in CSV in the Output
 ### folder, detailing which taxa were recognized and which were not
@@ -39,7 +39,7 @@ diat_loadData <- function(species_df, isRelAb=FALSE, maxDistTaxa=2, resultsPath)
     species_df <- as.data.frame(read.csv(file.choose())) #select the data matrix with a dialog box
   }
 
-  #Results folder
+  #Choose a Results folder
   if(missing(resultsPath)) {
     print("Select Results folder")
     resultsPath <- choose.dir(default = "", caption = "Select folder for your Results")
@@ -47,27 +47,26 @@ diat_loadData <- function(species_df, isRelAb=FALSE, maxDistTaxa=2, resultsPath)
   if (is.na(resultsPath)){stop("Calculations cancelled, no folder selected")}
   print("Result folder selected")
 
-  #check for duplicate species
+  #Check for duplicate species
   if("species" %in% colnames(species_df)) {
     print("Checking for duplicates")
     testdupl <- species_df[,"species"]
     if (length(testdupl[duplicated(testdupl)])> 0){ #check for duplicates
+      #duplicates found
       print("Duplicate species found in input data")
       testdupl[duplicated(testdupl)] #show duplicates
       stop("Cancelling.Duplicate species found in input data") #abort script
+    } else {
+      print("No duplicate species found in input data")
     }
-    speciesrow <- species_df[,"species"]
-    row.names(species_df) <- speciesrow  #converts the species names to rownames
+    row.names(species_df) <- species_df[,"species"]  #converts the species names to rownames
     species_df<- species_df[ , !(names(species_df) == "species")] #removes the species names column
   } else {
     #check if species names are in the first column
     stop("Calculations cancelled. File should contain a column named 'species' with the species list")
   }
-  #Reorder alphabetically
-  #species_df <- species_df[order(row.names(species_df)),] #reorder alphabetically
 
-
-  #Find empty samples
+  #Find and report any empty samples
   colvector <- c()
   for (i in 1:ncol(species_df)){
     if (is.numeric(species_df[,i])){
@@ -84,7 +83,7 @@ diat_loadData <- function(species_df, isRelAb=FALSE, maxDistTaxa=2, resultsPath)
 
   }
 
-  #If the acronym column exists, moves it to the back
+  #If the acronym column exists, removes it (for compatibility issues)
   acrocol <- which(colnames(species_df)=="acronym") #finds the acronym column
   if (length(acrocol) != 0) {
     species_df1 <- species_df[,-acrocol] #removes the acronym column if it exists
@@ -94,89 +93,73 @@ diat_loadData <- function(species_df, isRelAb=FALSE, maxDistTaxa=2, resultsPath)
     sampleNames <- colnames(species_df) #Saves the names of the samples
   }
 
-
-
   #SINCE THE ACRONYM COLUMN WAS REMOVED, WE HAVE TO CREATE A new_species COLUMN
+
   species_df$new_species <- NA
   species_df$new_species <- rownames(species_df)
 
   #removes NA in abundance data
   species_df[is.na(species_df)] <- 0
 
-  ########## LINK WITH DIAT.BARCODE DATABASE
-  #internal 'Diat.Barcode' number
-  intversion <- "9"
-  #get version number of latest 'Diat.Barcode'
-  dic <- read.csv("http://www.francoiskeck.fr/work/diatbarcode/dic_version.csv", header = TRUE, stringsAsFactors = FALSE)
-  if (exists("dic")){
-    #is able to check the version
-    version <- dic[dic$Flavor =="original",]
-    version <- version$Version[which.max(as.numeric(as.POSIXlt(version$Date, format = "%d-%m-%Y")))]
 
-    #compare both. If updates are needed, attempt them
-    if (version == intversion){
-      #updates are not needed
-      print ("No updates needed for the 'Diat.barcode' database. Proceeding!")
-      #load("data/dbc_offline.RData") ##takes the internal database
-      #dbc <- dbc_offline
-      dbc <- diathor::dbc_offline
+  ########## LINK WITH DIAT.BARCODE DATABASE (v.0.0.8)
+  getDiatBarcode <- diathor::diat_getDiatBarcode() #function that gets the Diat.Barcode database
+  ecodata <- as.data.frame(getDiatBarcode[1]) #ecodata
+  taxaList <- as.data.frame(getDiatBarcode[2]) #taxaList: diat_getDiatBarcode uses the taxaList() function to build a single list with all taxa. This is the result
 
-    } else {
-      #updates are needed
-      ########--------  Diat.Barcode download attempt. If it fails, tries to use internal database
-      ## WARNING: CRAN package does not auto-update the Diat.Barcode database
-      print("The diatom database in DiaThor is out of date")
+  #Trim blank spaces from input data
+  rownames(species_df) <- trimws(rownames(species_df))
 
-      ###### THIS SECTION IS FOR THE CRAN PROJECT ONLY
-      print("The CRAN version of the package does not auto-update the internal database. But the GitHub version does!")
-      print("Using internal database, 'Diat.barcode' v.9.0 published on 14-09-2020")
-      dbc <- diathor::dbc_offline
-      ###### END OF CRAN VERSION
+  ###### START NAME CHECK
 
-      ###### THIS SECTION IS FOR THE GITHUB PROJECT ONLY
-      # print("Attempting to download diat.barcode from website")
-      # dbc <- diatbarcode::get_diatbarcode(version = "last") #loads the latest version of diat.barcode
-      # if (exists("dbc")){ #it if was able to download the new version, proceed
-      #   print("Latest version of Diat.barcode succesfully downloaded. Remember to credit accordingly!")
-      # } else { #it if was unable to download the new version, use the internal database
-      #   print("Latest version of Diat.barcode cannot be downloaded")
-      #   print("Using internal database, Diat.barcode v.8.1 published on 10-06-2020. It might need to be updated")
-      #   #load("data/dbc_offline.RData") ##takes the internal database
-      #   dbc <- dbc_offline
-      # }
-      ###### END OF GITHUB VERSION
+  #PROGRESS BAR - SEARCH SPECIES FOR POSSIBLE MISSPELLS
+  pb <- txtProgressBar(min = 1, max = nrow(species_df), style = 3)
+  #macthing function, uses stringdist package
+  for (i in 1:nrow(species_df)){
+    #get the species name
+    spname <- row.names(species_df)[i]
+    #remove double spaces and trim blank characters
+    spname <- gsub("(?<=[\\s])\\s*|^\\s+|\\s+$", "", spname, perl = TRUE)  # Remove spaces
 
+    #Species that have "var." or "fo.", the period should be followed by a space. If not, add it
+    getPeriods <- lapply(strsplit(spname, ''), function(x) which(x == '.'))
+    #if there is a period
+    if (length(getPeriods[[1]]) > 0){
+      for (j in 1:length(getPeriods)){
+        #get index
+        periodIndex <- getPeriods[[j]]
+        if (substr(spname, periodIndex +1, periodIndex+1)!= " "){
+          #if no space after the period, add it
+          spname <- paste(substring(spname, c(1,periodIndex+1), c(periodIndex-1,nchar(spname))), collapse=". ")
+        }
+      }
     }
-  } else {
-    print("Latest version of 'Diat.barcode' unknown")
-    print("Using internal database, 'Diat.barcode' v.9.0 published on 14-09-2020")
-    dbc <- diathor::dbc_offline
+
+    #then correct the species name for all future uses
+    row.names(species_df)[i] <- spname
+
+    #searches heuristically for the species name agains the full list of taxaList (all taxa in all internal databases)
+    searchvectr <- taxaList[stringdist::ain(tolower(taxaList[,"species"]),tolower(spname), maxDist=maxDistTaxa, matchNA = FALSE),] #seaches species by species
+    if (length(searchvectr)==0){ #if species not found in any db
+      print(paste("Taxon not found in any internal database:", spname))
+    }
+    #update progressbar
+    setTxtProgressBar(pb, i)
   }
-  ### Double checks that database got loaded correctly or cancels alltogether
-  if (exists("dbc")){
-  } else { #if everything fails, cancels
-    stop("Database could not be downloaded or retrieved from internal package. Cancelling calculations")
-  }
-  ########## END LINK WITH DIAT.BARCODE DATABASE
-
-  #Remove duplicate by field "species" in diat.barcode
-  dbc2 <- as.data.frame(dbc[!duplicated(dbc[,"species"]),]) #transforms dbc to a dataframe
-  ecodata <- dbc2[which(colnames(dbc2)=="species" ):ncol(dbc2)] #keeps only from the "species" column onwards, to keep the ecological data
-
-
-
+  #close progressbar
+  close(pb)
 
+  ###### END NAME CHECK
 
   ########-------- MATCHES AND BINDS DATA SETS
   # NOTE: if several species match with the same name in the diat.barcode database (multiple strains), it keeps the
   # first occurence. Ecological indices usually match in all strains of the same species
   #fuzzy matching
 
-
-
   #SEARCH
-  rownames(species_df) <- trimws(rownames(species_df))
+  #creates empty data frame taxaInSp
   taxaInSp <- as.data.frame(matrix(nrow=nrow(species_df), ncol = (ncol(species_df)+ncol(ecodata))))
+
   #species_df section
   taxaInSp[1:nrow(species_df),1:ncol(species_df)] <- species_df[1:nrow(species_df),1:ncol(species_df)] #copies species_df into taxaInSp
   colnames(taxaInSp)[1:ncol(species_df)] <- colnames(species_df) #copies column names
@@ -186,8 +169,10 @@ diat_loadData <- function(species_df, isRelAb=FALSE, maxDistTaxa=2, resultsPath)
   #ecodata_section
   lastcolspecies_df <-  which(colnames(taxaInSp)=="new_species") #gets last column of taxaInSp with species_df data
   colnames(taxaInSp)[(lastcolspecies_df+1):(ncol(taxaInSp)-1)] <- colnames(ecodata) #copies column names
+
   #PROGRESS BAR
   pb <- txtProgressBar(min = 1, max = nrow(taxaInSp), style = 3)
+  #macthing function, uses stringdist package
   for (i in 1:nrow(taxaInSp)){
     searchvectr <- ecodata[stringdist::ain(ecodata[,"species"],row.names(species_df)[i], maxDist=maxDistTaxa, matchNA = FALSE),] #seaches species by species
     if (nrow(searchvectr)==1){ #if it finds only one species, add that
@@ -195,8 +180,14 @@ diat_loadData <- function(species_df, isRelAb=FALSE, maxDistTaxa=2, resultsPath)
       taxaInSp[i,"recognizedSp"] <- searchvectr$species
     } else if (nrow(searchvectr)>1){ #if it finds multiple species, keeps the one with the lower distance
       searchvectr <- searchvectr[which(stringdist::stringdist(searchvectr$species, row.names(species_df)[i]) == min(stringdist::stringdist(searchvectr$species, row.names(species_df)[i]))),]
-      if (nrow(searchvectr) > 1) { #still finds more than one with the same lower distance, keeps the first
-        searchvectr <- searchvectr[1,]
+      if (nrow(searchvectr) > 1) { #still finds more than one with the same lower distance, creates a majority consensus for each column
+        consensus_row <- matrix(nrow = 1, ncol = ncol(searchvectr))
+        for (j in 1:ncol(searchvectr)){
+          consensus_row[1,j] <- names(which.max(table(searchvectr[,j])))
+        }
+        searchvectr_names <- colnames(searchvectr)
+        searchvectr <- as.data.frame(consensus_row)
+        colnames(searchvectr) <- searchvectr_names
       }
       taxaInSp[i,(lastcolspecies_df + 1):(ncol(taxaInSp)-1)] <- searchvectr
       taxaInSp[i,"recognizedSp"] <- searchvectr$species
@@ -211,16 +202,11 @@ diat_loadData <- function(species_df, isRelAb=FALSE, maxDistTaxa=2, resultsPath)
   close(pb)
   #END NEW SEARCH
 
-
-
-
   taxaInEco <- taxaInSp
   taxaIn <- species_df #dataframe to be exported for indices
   taxaIncluded <- as.data.frame(rownames(taxaInEco)[which(taxaInEco$recognizedSp != "Not found")])
   taxaExcluded <- as.data.frame(rownames(taxaInEco)[which(taxaInEco$recognizedSp == "Not found")])
 
-
-
   #remove the updated species col  and move it to the back in both taxaIn
   newspecies_col <- taxaIn$new_species
   newspecies_col <- replace(newspecies_col, newspecies_col=="0", NA)
@@ -256,13 +242,11 @@ diat_loadData <- function(species_df, isRelAb=FALSE, maxDistTaxa=2, resultsPath)
   }
   #Exports included taxa in morphology analyses
   colnames(taxaIncluded) <- "Eco/Morpho"
- # write.csv(taxaIncluded, paste(resultsPath,"\\Taxa included.csv", sep=""))
   write.csv(taxaIncluded, file.path(resultsPath,"Taxa included.csv"))
   print(paste("Number of taxa recognized for morphology:", nrow(taxaIncluded), "-- Detailed list in 'Taxa included.csv'"))
 
   #also makes a matrix for all the taxa left out, for the user to review
   colnames(taxaExcluded) <- "Eco/Morpho"
-  #write.csv(taxaExcluded, paste(resultsPath,"\\Taxa excluded.csv", sep=""))
   write.csv(taxaExcluded, file.path(resultsPath,"Taxa excluded.csv"))
   print(paste("Number of taxa excluded for morphology:", nrow(taxaExcluded), "-- Detailed list in 'Taxa excluded.csv'" ))
 
@@ -273,15 +257,10 @@ diat_loadData <- function(species_df, isRelAb=FALSE, maxDistTaxa=2, resultsPath)
   precisionmatrix <- as.data.frame(sampleNames)
   names(precisionmatrix)[names(precisionmatrix)=="sampleNames"] <- "Sample"
 
-  #write.csv(precisionmatrix, paste(resultsPath,"\\Precision.csv", sep=""))
-  write.csv(precisionmatrix, file.path(resultsPath, "Precision.csv"))
+  write.csv(precisionmatrix, file.path(resultsPath, "num_taxa.csv"))
 
   #gets the column named "new_species" everything before that column should be a sample with abundance data
   lastcol = which(colnames(taxaIn)=="new_species")
-  #
-  # #gets sample names
-  # sampleNames <- colnames(taxaIn[1:(lastcol-1)])
-
 
 
   #CREATES A RELATIVE ABUNDANCE MATRIX AS WELL FOR THOSE INDICES THAT USE IT
@@ -297,6 +276,11 @@ diat_loadData <- function(species_df, isRelAb=FALSE, maxDistTaxa=2, resultsPath)
   } else {
     taxaInRA <- taxaIn
   }
+  #converts to double taxaInRA
+  lastcol <- which(colnames(taxaInRA)=="new_species")
+  for (i in 1:(lastcol-1)){
+    taxaInRA[,i] <- as.double(taxaInRA[,i])
+  }
 
 
   #CREATES THE EXPORT PRODUCTS
