@@ -103,6 +103,11 @@ diat_lobo <- function(resultLoad, maxDistTaxa = 2){
             vvalue = NA
             svalue = NA
           }
+        }  else {
+          # length(spsplit[[1]]) =<1
+          #species not found, make everything NA
+          vvalue = NA
+          svalue = NA
         }
       }
 
