@@ -108,6 +108,11 @@ diat_tdi <- function(resultLoad, maxDistTaxa = 2){
               vvalue = NA
               svalue = NA
           }
+        }  else {
+          # length(spsplit[[1]]) =<1
+          #species not found, make everything NA
+          vvalue = NA
+          svalue = NA
         }
       }
     }
